using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using UnityEngine;
using General.Mediator;
using UnityEngine.Purchasing;
#if FIREBASE_ANALYTICS || APP_METRICA
using SP;
using SP.Analytics;
#endif
#if ADDRESSABLES
using UnityEngine.AddressableAssets;
#endif


namespace IAP
{
    internal class IAPProvider : IStoreListener, IIAP
    {
        #region Fields
        
        private const string AssetReferenceIsNull = "Asset reference is null!";
        private const string PathResourcesIsNullOrEmpty = "Path resources is null or empty!";
        private const string TextJsonIsNullOrEmpty = "Text json is null or empty!";
        private const string DataIsNullOrEmpty = "Data is null or empty!";
        private const string EventName = "Purchase";
        private IStoreController controller;
        private IExtensionProvider extensions;
        private Dictionary<string, ProductConfig> configs;
        public event Action<ProductCollection> Initialized;
        public event Action<string> InitializeFailed;
        public event Action<string> PurchaseSuccess;
        public event Action<PurchaseEventArgs> PurchaseSuccessWithArgs;
        public event Action<string> PurchaseFailed;

        #endregion

        #region Properties
        
        public bool IsInitialized => controller is { } && extensions is { };
        public bool Logger { get; set; } = true;
        public Dictionary<string, Action<Product, Action<Product>>> PurchaseOrder { get; private set; } = 
            new Dictionary<string, Action<Product, Action<Product>>>();

        #endregion

        #region Methods
        
        public void InitializeViaConfig(ProductConfigWrapper productConfigWrapper)
        {
            if (IsInitialized) return;
            Initialize(productConfigWrapper, ConfigurationBuilderPurchas());
        }
        
        public void InitializeViaResources(string pathResources)
        {
            if(IsInitialized) return;
            if (string.IsNullOrEmpty(pathResources))
            {
                Debug.LogError(PathResourcesIsNullOrEmpty);
                return;
            }
            var data = Resources.Load<TextAsset>(pathResources)?.text;
            if(IsNullOrEmpty(data))
                return;
            Initialize(data, ConfigurationBuilderPurchas());
        }

        public void InitializeViaResources(TextAsset textJson)
        {
            if (textJson is null)
            {
                Debug.LogError(TextJsonIsNullOrEmpty);
                return;
            }
            Initialize(textJson.text, ConfigurationBuilderPurchas());
        }

#if ADDRESSABLES
        public async void InitializeViaAddressables(AssetReference assetReference)
        {
            if(IsInitialized) return;
            if (assetReference is null)
            {
                Debug.LogError(AssetReferenceIsNull);
                return;
            }
            var data = await LoadConfigs(assetReference);
            if(IsNullOrEmpty(data))
                return;
            Initialize(data,ConfigurationBuilderPurchas());
        }
#endif

        public void StartPurchase(string id, Action<Product, Action<Product>> verificationPurchase = null)
        {
            if (IsInitialized)
            {
                if(verificationPurchase != null) 
                    PurchaseOrder.Add(id,verificationPurchase);
                controller.InitiatePurchase(id);
            }
            else
            {
                Debug.LogWarning("Purchase is not Initialized");
            }
        }

        #region IStoreListener

        public void OnInitialized(IStoreController controller, IExtensionProvider extensions)
        {
            this.controller = controller;
            this.extensions = extensions;
#if UNITY_IOS
            extensions.GetExtension<IAppleExtensions>().RestoreTransactions(result => 
            {
                Debug.Log($"The purchase restoration process {result}");
            });
#endif
            Initialized?.Invoke(controller.products);
            if(Logger) 
                Debug.Log("Purchase Initialized"); // TODO : Connect With Log System
            SendAnalytics("Initialize", "Success");
        }

        public void OnInitializeFailed(InitializationFailureReason error)
        {
            InitializeFailed?.Invoke(error.ToString());
            if(Logger) 
                Debug.Log($"Purchase InitializeFailed {error}"); // TODO : Connect With Log System
            SendAnalytics("Initialize", $"Failed {error.ToString()}");
        }

        public PurchaseProcessingResult ProcessPurchase(PurchaseEventArgs purchaseEvent)
        {
            if (PurchaseOrder.TryGetValue(purchaseEvent.purchasedProduct.definition.id,out Action<Product, Action<Product>> action))
            {
                action.Invoke(purchaseEvent.purchasedProduct,ConfirmPendingPurchase);
                return PurchaseProcessingResult.Pending;
            }
            PurchaseSuccess?.Invoke(purchaseEvent.purchasedProduct.definition.id);
            PurchaseSuccessWithArgs?.Invoke(purchaseEvent);
            SuccessPurchasing(purchaseEvent.purchasedProduct);
            return PurchaseProcessingResult.Complete;
        }

        public void OnPurchaseFailed(Product product, PurchaseFailureReason failureReason)
        {
            if(Logger) 
                Debug.Log(
                $"Purchase Failed, product id {product.definition.id}, failure reason {failureReason}, transaction {product.transactionID}\n" +
                $"Available To Purchase {product.availableToPurchase}\n" +  $"Metadata |  Price {product.metadata.localizedPrice} Description {product.metadata.localizedDescription}");
            // TODO : Connect With Log System
            
            PurchaseFailed?.Invoke(product.definition.id);
            
            SendAnalytics(product.definition.id,$"Failure reason {failureReason}");
        }

        #endregion

#if ADDRESSABLES
        private async Task<string> LoadConfigs(AssetReference assetReferenceProductsConfig)
        {
            //TODO : Not Implemented
            Debug.Log("Addressables not installed" );
            throw new NotImplementedException();
        }
#endif

        private void ConfirmPendingPurchase(Product product)
        {
            controller.ConfirmPendingPurchase(product);
            SuccessPurchasing(product);
            PurchaseSuccess?.Invoke(product.definition.id);
        }

        private void SuccessPurchasing(Product product)
        {
            if(Logger) 
                Debug.Log($"Purchase success {product.definition.id}");// TODO : Connect With Log System
            
            SendAnalytics(product.definition.id,"Purchase success");
        }

        private void Initialize(string data,ConfigurationBuilder builder)
        {
            configs = JsonUtility.FromJson<ProductConfigWrapper>(data).Configs.ToDictionary(x => x.Id, x => x);
            if(Logger) 
                Debug.Log(data);
            InitializeInternal(builder);
        }

        private void Initialize(ProductConfigWrapper productConfigWrapper, ConfigurationBuilder builder)
        {
            configs = productConfigWrapper.Configs.ToDictionary(x => x.Id, x => x);
            InitializeInternal(builder);
        }

        private void InitializeInternal(ConfigurationBuilder builder)
        {
            foreach (var item in configs.Values)
            {
                builder.AddProduct(item.Id, item.Type);
                if(Logger)
                    Debug.Log($"id {item.Id} | type {item.Type}");
            }
            UnityPurchasing.Initialize(this, builder);
        }

        private ConfigurationBuilder ConfigurationBuilderPurchas()
        {
            var builder = ConfigurationBuilder.Instance(StandardPurchasingModule.Instance());
            builder.Configure<IGooglePlayConfiguration>().SetServiceDisconnectAtInitializeListener(() =>
            {
                Debug.Log("Unable to connect to the Google Play Billing service.");
            });
            return builder;
        }
        
        private bool IsNullOrEmpty(string data)
        {
            if (!string.IsNullOrEmpty(data)) return false;
            Debug.LogError(DataIsNullOrEmpty);
            return true;
        }
        
        private void SendAnalytics(string title,string message)
        {
#if FIREBASE_ANALYTICS || APP_METRICA
            if (MC.Instance.GetSingleComponent(this, out AnalyticsWrapper analytics))
            {
               var parameters = new Dictionary<string, object> {{title, message}};
               analytics.ReportEvent(EventName,parameters);
            }
#endif
        }
        
        #endregion
    }
}